# -*- coding: utf-8 -*-
"""
/***************************************************************************
 QGribDownloaderDockWidget
                                 A QGIS plugin
 download gribs from opengribs.org
 Generated by Plugin Builder: http://g-sherman.github.io/Qgis-Plugin-Builder/
                             -------------------
        begin                : 2019-02-26
        git sha              : $Format:%H$
        copyright            : (C) 2019 by enrico ferreguti
        email                : enricofer@gmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

import os
import urllib
import tempfile
import json

from PyQt5 import QtGui, QtWidgets, uic
from PyQt5.QtCore import QUrl, pyqtSignal, QJsonDocument
from PyQt5.QtWidgets import QApplication, QPushButton, QFileDialog
from PyQt5.QtNetwork import QNetworkRequest

from qgis.core import (
    QgsCoordinateReferenceSystem,
    QgsNetworkAccessManager,
    QgsMeshLayer,
    QgsProject,
    QgsRectangle,
    Qgis,
    QgsWkbTypes,
    QgsGeometry
)

from qgis.gui import QgsMessageBar, QgsRubberBand

from .QGribDownloader_defs import (
    atm_models,
    atm_model_capabilities,
    surface_data_decode,
    resolution_decode,
    altitude_decode,
    intv_decode,
    cyc_decode,
    days_decode,
    wave_models,
    wave_model_capabilities,
    wavedata_decode,
)

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'QGribDownloader_dockwidget_base.ui'))

STATUS_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'QGribDownloader_server_status.ui'))

def build_url(baseurl, path, args_dict):
    # Returns a list in the structure of urlparse.ParseResult
    url_parts = list(urlparse.urlparse(baseurl))
    url_parts[2] = path
    url_parts[4] = urllib.urlencode(args_dict)
    return urlparse.urlunparse(url_parts)

class QGribDownloaderServerStatusDialog(QtWidgets.QDialog, STATUS_CLASS):

    def __init__(self, parent=None):
        """Constructor."""
        super(QGribDownloaderServerStatusDialog, self).__init__(parent)
        self.setupUi(self)

    def updateServerStatus(self):
        manager = QgsNetworkAccessManager.instance()
        request = QNetworkRequest()
        request.setUrl(QUrl("http://grbsrv.opengribs.org/getstatus2.php"))
        rawReplyObject = manager.blockingGet(request)
        j = QJsonDocument.fromJson(rawReplyObject.content())
        replyObject = j.toVariant()
        self.status_report.setPlainText(json.dumps(replyObject,indent=3))

    @staticmethod
    def getStatus():
        dialog = QGribDownloaderServerStatusDialog()
        dialog.updateServerStatus()
        result = dialog.exec_()

class QGribDownloaderDockWidget(QtWidgets.QDockWidget, FORM_CLASS):

    closingPlugin = pyqtSignal()

    def __init__(self, iface, parent=None):
        """Constructor."""
        super(QGribDownloaderDockWidget, self).__init__(parent)
        # Set up the user interface from Designer.
        # After setupUI you can access any designer object by doing
        # self.<objectname>, and you can use autoconnect slots - see
        # http://qt-project.org/doc/qt-4.8/designer-using-a-ui-file.html
        # #widgets-and-dialogs-with-auto-connect
        self.setupUi(self)
        self.iface = iface
        self.context = QgsRubberBand(self.iface.mapCanvas(), QgsWkbTypes.PolygonGeometry)
        self.context.setFillColor(QtGui.QColor("#000000ff"))
        self.context.setStrokeColor(QtGui.QColor("#fdb100"))
        self.context.setWidth(2)
        self.modelsComboBox.addItems(atm_models)
        self.wavemodelComboBox.addItems(wave_models)
        self.modelsComboBox.currentIndexChanged.connect(self.model_changed)
        self.download_pushButton.clicked.connect(self.download)
        self.status_pushButton.clicked.connect(self.status)
        self.mExtentGroupBox.setMapCanvas(self.iface.mapCanvas())
        self.mExtentGroupBox.setCurrentExtent(self.iface.mapCanvas().extent(),self.iface.mapCanvas().mapSettings().destinationCrs())
        self.mExtentGroupBox.setOriginalExtent(QgsRectangle(6,36,18,47),QgsCoordinateReferenceSystem(4326))
        self.mExtentGroupBox.setOutputCrs(QgsCoordinateReferenceSystem(4326))
        self.mExtentGroupBox.extentChanged.connect(self.drawContext)
        self.model_changed(0)

        #mButtonDrawOnCanvas causes crash
        for button in self.mExtentGroupBox.findChildren(QPushButton):
            if button.objectName() == "mButtonDrawOnCanvas":
                button.hide()


    def model_changed(self,current_model):
        self.capabilities = atm_model_capabilities[self.modelsComboBox.currentText()]
        for field in ["altitude", "surface_data"]:
            field_decode = globals()[field +"_decode"]
            for attr in field_decode.keys():
                field_widget = getattr(self, "_checkBox_" + attr.replace(";",""))
                field_widget.setEnabled(attr in self.capabilities[field])
        for field in ["resolution", "intv", "cyc", "days"]:
            field_widget =  getattr(self,field +"_comboBox")
            field_widget.clear()
            field_decode = globals()[field +"_decode"]
            for attr in self.capabilities[field]:
                field_widget.addItem(field_decode[attr],attr)

    def status(self):
        QGribDownloaderServerStatusDialog.getStatus()

    def download(self):

        self.iface.messageBar().pushMessage("QGribDownloader", "Sending request to opengribs.org", level=Qgis.Info, duration=99)
        QApplication.processEvents()
        ext = self.mExtentGroupBox.outputExtent()

        url_params = {
            "osys": "Unknown",
            "ver": atm_model_capabilities["version"],
            "la1": str(ext.yMinimum()),
            "la2": str(ext.yMaximum()),
            "lo1": str(ext.xMinimum()),
            "lo2": str(ext.xMaximum()),
            "model": self.capabilities["parameter"]+self.resolution_comboBox.currentData(),
            "wmdl": wave_model_capabilities[self.wavemodelComboBox.currentText()]["wmdl"]
        }

        for param in ["intv", "days", "cyc"]:
            wdgt = getattr(self,param + "_comboBox")
            url_params[param] = str(wdgt.currentData())

        atm_params = ""
        for param in list(surface_data_decode.keys())+list(altitude_decode.keys()):
            wdgt = getattr(self, "_checkBox_" + param.replace(";",""))
            if wdgt.isEnabled():
                if wdgt.isChecked():
                    atm_params += param

        wave_params = ""
        for param in ["s;", "H;D;P;", "h;d;p;"]:
            wdgt = getattr(self, "_checkBox__" + param.replace(";",""))
            if wdgt.isEnabled():
                if wdgt.isChecked():
                    wave_params += param

        url_params["par"] = atm_params
        url_params["wpar"] = wave_params

        url = "http://grbsrv.opengribs.org/getmygribs2.php/?" + urllib.parse.urlencode(url_params)

        manager = QgsNetworkAccessManager.instance()
        request = QNetworkRequest()
        request.setUrl(QUrl(url))
        QApplication.processEvents()
        rawReplyObject = manager.blockingGet(request)
        j = QJsonDocument.fromJson(rawReplyObject.content())
        replyObject = j.toVariant()
        if replyObject["status"]:
            mesh_name = QUrl(replyObject["message"]["url"]).fileName()
            targetFolder = QFileDialog.getExistingDirectory(self, self.tr("Save Grib"))
            targetFile = os.path.join(targetFolder, mesh_name)
            request.setUrl(QUrl(replyObject["message"]["url"]))
            replyObject = manager.blockingGet(request)
            with open(targetFile, 'wb') as f:
                f.write(replyObject.content())
            self.status_label.setText("Downloaded: "+targetFile)
            self.iface.messageBar().clearWidgets()
            self.iface.messageBar().pushMessage("QGribDownloader", "Downloaded: " + targetFile, level=Qgis.Success, duration=9999)
            meshLayer = QgsMeshLayer(targetFile, mesh_name, "mdal")
            QgsProject.instance().addMapLayer(meshLayer)
            self.iface.mapCanvas().setDestinationCrs(meshLayer.crs())
        else:
            self.iface.messageBar().clearWidgets()
            self.iface.messageBar().pushMessage("QGribDownloader", "Download error: " + replyObject["message"], level=Qgis.Critical, duration=5)


    def show(self):
        self.drawContext(self.mExtentGroupBox.currentExtent())
        super(QGribDownloaderDockWidget, self).show()

    def hide(self):
        self.context.reset()
        super(QGribDownloaderDockWidget, self).hide()

    def updateExtent(self):
        self.mExtentGroupBox.setOutputExtentFromCurrent()

    def drawContext(self,ext):
        ext = self.mExtentGroupBox.outputExtent()
        ext_geom = QgsGeometry.fromWkt(ext.asWktPolygon())
        self.context.reset()
        self.context.addGeometry(ext_geom,self.mExtentGroupBox.outputCrs())


    def closeEvent(self, event):
        self.closingPlugin.emit()
        event.accept()
